@@ -167,7 +167,7 @@ buf_empty_p (struct buffer *buf)
 
 
 
-# ifdef SERVER_FLOWCONTROL
+# if defined (SERVER_FLOWCONTROL) || defined (PROXY_SUPPORT)
 /*
  * Count how much data is stored in the buffer..
  * Note that each buffer is a xmalloc'ed chunk BUFFER_DATA_SIZE.
@@ -183,7 +183,7 @@ buf_count_mem (struct buffer *buf)
 
     return mem;
 }
-# endif /* SERVER_FLOWCONTROL */
+# endif /* SERVER_FLOWCONTROL || PROXY_SUPPORT */
 
 
 
@@ -506,7 +506,7 @@ buf_append_data (struct buffer *buf, struct buffer_data *data,
 
 
 
-#ifdef PROXY_SUPPORT
+# ifdef PROXY_SUPPORT
 /* Copy data structures and append them to a buffer.
  *
  * ERRORS
@@ -553,7 +553,7 @@ buf_free_data (struct buffer *buffer)
     free_buffer_data = buffer->data;
     buffer->data = buffer->last = NULL;
 }
-#endif /* PROXY_SUPPORT */
+# endif /* PROXY_SUPPORT */
 
 
 
@@ -1988,12 +1988,15 @@ fd_buffer_flush (void *closure)
 
 
 
+static struct stat devnull;
+static int devnull_set = -1;
+
 /* The buffer block function for a buffer built on a file descriptor.  */
 static int
 fd_buffer_block (void *closure, bool block)
 {
     struct fd_buffer *fb = closure;
-#if defined (F_GETFL) && defined (O_NONBLOCK) && defined (F_SETFL)
+# if defined (F_GETFL) && defined (O_NONBLOCK) && defined (F_SETFL)
     int flags;
 
     flags = fcntl (fb->fd, F_GETFL, 0);
@@ -2005,11 +2008,39 @@ fd_buffer_block (void *closure, bool block)
     else
 	flags |= O_NONBLOCK;
 
-    if (fcntl (fb->fd, F_SETFL, flags) < 0 && errno != ENODEV)
-	/* BSD returns ENODEV when we try to set block/nonblock on /dev/null.
+    if (fcntl (fb->fd, F_SETFL, flags) < 0)
+    {
+	/*
+	 * BSD returns ENODEV when we try to set block/nonblock on /dev/null.
+	 * BSDI returns ENOTTY when we try to set block/nonblock on /dev/null.
 	 */
-	return errno;
-#endif /* F_GETFL && O_NONBLOCK && F_SETFL */
+	struct stat sb;
+	int save_errno = errno;
+	bool isdevnull = false;
+
+	if (devnull_set == -1)
+	    devnull_set = stat ("/dev/null", &devnull);
+
+	if (devnull_set >= 0)
+	    /* Equivalent to /dev/null ? */
+	    isdevnull = (fstat (fb->fd, &sb) >= 0
+			 && sb.st_dev == devnull.st_dev
+			 && sb.st_ino == devnull.st_ino
+			 && sb.st_mode == devnull.st_mode
+			 && sb.st_uid == devnull.st_uid
+			 && sb.st_gid == devnull.st_gid
+			 && sb.st_size == devnull.st_size
+			 && sb.st_blocks == devnull.st_blocks
+			 && sb.st_blksize == devnull.st_blksize);
+	if (isdevnull)
+	    errno = 0;
+	else
+	{
+	    errno = save_errno;
+	    return errno;
+	}
+    }
+# endif /* F_GETFL && O_NONBLOCK && F_SETFL */
 
     fb->blocking = block;
 
