@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -23,20 +18,18 @@
 
 #include "cvs.h"
 
-static int find_dirs PROTO((char *dir, List * list, int checkadm,
-			    List *entries));
-static int find_rcs PROTO((char *dir, List * list));
-static int add_subdir_proc PROTO((Node *, void *));
-static int register_subdir_proc PROTO((Node *, void *));
+static int find_dirs (char *dir, List * list, int checkadm,
+			    List *entries);
+static int find_rcs (char *dir, List * list);
+static int add_subdir_proc (Node *, void *);
+static int register_subdir_proc (Node *, void *);
 
 /*
  * add the key from entry on entries list to the files list
  */
-static int add_entries_proc PROTO((Node *, void *));
+static int add_entries_proc (Node *, void *);
 static int
-add_entries_proc (node, closure)
-     Node *node;
-     void *closure;
+add_entries_proc (Node *node, void *closure)
 {
     Node *fnode;
     List *filelist = closure;
@@ -59,11 +52,7 @@ add_entries_proc (node, closure)
    list).  */
 
 List *
-Find_Names (repository, which, aflag, optentries)
-    char *repository;
-    int which;
-    int aflag;
-    List **optentries;
+Find_Names (char *repository, int which, int aflag, List **optentries)
 {
     List *entries;
     List *files;
@@ -94,7 +83,8 @@ Find_Names (repository, which, aflag, optentries)
 	/* search the repository */
 	if (find_rcs (repository, files) != 0)
 	{
-	    error (0, errno, "cannot open directory %s", repository);
+	    error (0, errno, "cannot open directory %s",
+		   primary_root_inverse_translate (repository));
 	    goto error_exit;
 	}
 
@@ -108,7 +98,8 @@ Find_Names (repository, which, aflag, optentries)
 		&& !existence_error (errno))
 		/* For now keep this a fatal error, seems less useful
 		   for access control than the case above.  */
-		error (1, errno, "cannot open directory %s", dir);
+		error (1, errno, "cannot open directory %s",
+		       primary_root_inverse_translate (dir));
 	    free (dir);
 	}
     }
@@ -127,9 +118,7 @@ Find_Names (repository, which, aflag, optentries)
  */
 
 static int
-add_subdir_proc (p, closure)
-     Node *p;
-     void *closure;
+add_subdir_proc (Node *p, void *closure)
 {
     List *dirlist = closure;
     Entnode *entnode = p->data;
@@ -152,9 +141,7 @@ add_subdir_proc (p, closure)
 
 /*ARGSUSED*/
 static int
-register_subdir_proc (p, closure)
-     Node *p;
-     void *closure;
+register_subdir_proc (Node *p, void *closure)
 {
     List *entries = (List *) closure;
 
@@ -166,10 +153,7 @@ register_subdir_proc (p, closure)
  * create a list of directories to traverse from the current directory
  */
 List *
-Find_Directories (repository, which, entries)
-    char *repository;
-    int which;
-    List *entries;
+Find_Directories (char *repository, int which, List *entries)
 {
     List *dirlist;
 
@@ -257,9 +241,7 @@ Find_Directories (repository, which, entries)
  * containing the files which were found before the error occurred).
  */
 static int
-find_rcs (dir, list)
-    char *dir;
-    List *list;
+find_rcs (char *dir, List *list)
 {
     Node *p;
     struct dirent *dp;
@@ -306,11 +288,7 @@ find_rcs (dir, list)
  * error, in which case errno is set to indicate the error.
  */
 static int
-find_dirs (dir, list, checkadm, entries)
-    char *dir;
-    List *list;
-    int checkadm;
-    List *entries;
+find_dirs (char *dir, List *list, int checkadm, List *entries)
 {
     Node *p;
     char *tmp = NULL;
@@ -324,7 +302,7 @@ find_dirs (dir, list, checkadm, entries)
        a normal directory name.  */
     if (isabsolute (dir)
 	&& strncmp (dir, current_parsed_root->directory, strlen (current_parsed_root->directory)) == 0
-	&& ISDIRSEP (dir[strlen (current_parsed_root->directory)])
+	&& ISSLASH (dir[strlen (current_parsed_root->directory)])
 	&& strcmp (dir + strlen (current_parsed_root->directory) + 1, CVSROOTADM) == 0)
 	skip_emptydir = 1;
 
