@@ -1,8 +1,6 @@
 /*
- * Copyright (C) 2003-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 2003-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
+ *    Copyright (c) 2003, Derek Price, Ximbiot <http://ximbiot.com>,
+ *                        and the Free Software Foundation
  *
  *    You may distribute under the terms of the GNU General Public License
  *    as specified in the README file that comes with the CVS source
@@ -12,4 +10,7 @@
  * with other portions of CVS.
  */
 
+#ifndef HISTORY_H
+# define HISTORY_H
 #define ALL_HISTORY_REC_TYPES "TOEFWUPCGMAR"
+#endif /* HISTORY_H */
